# -*-python-*-

__package__    = "pocketMMap/toc.py"
__version__    = "0.2a"
__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/pocketMMap"
__date__       = "$Date: 2008/09/08 15:54:14 $"
__copyright__  = "Copyright (c) 2007-2008 Aaron Straup Cope. BSD license : http://www.modestmaps.com/license.txt"

import cairo
import Image

import math

class toc :

    def __init__ (self, width, height, margin=10) :
        self.page_height = height
        self.page_width = width
        self.margin = margin
        
        self.pages = []

    #
    #
    #
    
    def create (self, data) :

        data.sort(lambda m, n: cmp(m[0].lower(), n[0].lower()))
            
        (surface, ctx, x, y) = self.start_page()        

        sz_dot = self.measure_text(ctx, ".")
        
        for stuff in data:
            
            label = stuff[0]
            page = stuff[1]
            marker = stuff[2]

            sz_label = self.measure_text(ctx, label)
            
            loc = " %s / %s" % (page, marker)
            sz_loc = self.measure_text(ctx, loc)

            stop = self.page_width - (sz_loc[0] + self.margin)    

            while sz_label[0] > (stop - sz_loc[0]) :
                label = label[:-1]
                sz_label = self.measure_text(ctx, label)

            ctx.move_to(x, y)
            ctx.show_text(unicode(label))

            x = x + (sz_label[0] + (sz_dot[0] * 2))

            while x < stop :
                ctx.move_to(x, y)
                ctx.show_text(".")
                x = x + (sz_dot[0] * 2)
                
            ctx.move_to(x, y)
            ctx.show_text(loc)

            x = self.margin
            y = y + math.ceil(max(sz_label[1], sz_loc[1]) * 1.5)

            if y > (self.page_height - (self.margin * 2)) :
                self.write_page(surface)
                (surface, ctx, x, y) = self.start_page()    

        self.write_page(surface)
        return self.pages
        
    #
    #
    #
    
    def measure_text (self, ctx, txt):
        
        xbearing, ybearing, width, height, xadvance, yadvance = (
            ctx.text_extents(unicode(txt)))
        
        return (width, height)

    #
    #
    #

    def start_page (self) :

        surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, self.page_width, self.page_height)
        ctx = cairo.Context(surface)

        ctx.move_to(0, 0)
        ctx.rectangle(0, 0, self.page_width, self.page_height)
        ctx.set_source_rgb(255, 255, 255)
        ctx.fill()

        ctx.set_source_rgb(0, 0, 0)

        x = self.margin
        y = self.margin * 2

        legend = "page / marker"
        lg_bar = self.measure_text(ctx, legend)

        x = self.page_width - int(lg_bar[0] + (self.margin / 2))
        
        ctx.move_to(x, y)
        ctx.show_text(legend)

        x = self.margin
        y = y + int(lg_bar[1] * 3)

        ctx.move_to(x, y)

        return (surface, ctx, x, y)

    #
    #
    #
    
    def write_page (self, surface) :
        png = self.cairo2pil(surface)
        self.pages.append(png)

    #
    #
    #
    
    def cairo2pil(self, surface) :

        mode='RGBA'

        width = surface.get_width()
        height = surface.get_height()

        return Image.frombuffer(mode, (width, height), surface.get_data(), "raw", mode, 0, 1)

    #
    #
    #
    
if __name__ == '__main__' :

    data = [
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        ("label A", 1, "A"),
        ("spork", 127, "B"),
        ("label A", 1, "A"),
        ("spork", 3, "B"),
        
        ]

    t = toc(360, 200)
    pages = t.create(data)
